
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: BATTLENET_FONT_COLOR, BNConnected, BreakUpLargeNumbers, C_ChallengeMode, C_Club, C_GuildInfo, COMMUNITIES, COMMUNITIES_SETTINGS_MOTD_LABEL, CommunitiesFrame, DUNGEON_SCORE, Enum, GameTooltip, GUILD, IsTrialAccount, LABEL_NOTE, LEVEL, MEMBERS, NAME, next, NORMAL_FONT_COLOR, RANK, SetCVar, table, ToggleGuildFrame, type, UNAVAILABLE, ZONE

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display Guild")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI Guild", {
	type = "data source",
	text = "0",
	value = "0",
	label = "Guild",
	glyph = C.Enum.Glyph.UserSlash,
})
local Social = LibStub("SixSocial-1.0")
local Tooltip = LibStub("SixTooltip-1.0"):CreateTooltip({
	cellMarginH = C.TOOLTIP_CELL_MARGIN_H,
	cellMarginV = C.TOOLTIP_CELL_MARGIN_V,
	columnJustification = {"CENTER", "RIGHT", "LEFT", "LEFT", "LEFT", "LEFT", "RIGHT"},
	font = C.TEXT_REGULAR,
	headerFont = C.TEXT_LARGE,
	key = "SixUI Guild",
	numColumns = 7,
	separatorHeight = C.TOOLTIP_SEPARATOR_HEIGHT,
})

local col = nil
local GUILD_EVENT = C.IS_RETAIL and "SIXSOCIAL_CLUB_UPDATE" or "SIXSOCIAL_GUILD_UPDATE"
local list = Social.list
local rev = nil
local upper = Social.upper

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function GetNumOnline(members)
	local numOnline = 0

	for _, member in next, members do
		if member.status ~= Social.Enum.Status.Offline then
			numOnline = numOnline + 1
		end
	end

	return numOnline
end

local function SortMembersByColumn(a, b)
	if rev then a, b = b, a end
	local x, y = a[col], b[col]

	if (x and not y) or (not x and y) then
		return not not x
	elseif x ~= y and type(x) == "number" and type(y) == "number" then
		return x < y
	elseif x ~= y and type(x) == "string" and type(y) == "string" then
		return upper(x or "ZZZ") < upper(y or "ZZZ")
	else
		return a < b
	end
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self:RegisterMessage("SIXUI_CONFIG_CHANGED")
	self:LoadConfig()
	self.clubs = {}
end

function M:OnEnable()
	if not self:IsHooked("SetCVar") then
		self:SecureHook("SetCVar")
	end

	self:RegisterEvent("BN_CONNECTED", "Update")
	self:RegisterEvent("BN_DISCONNECTED", "Update")
	self:RegisterEvent("CHAT_DISABLED_CHANGE_FAILED", "Update")
	self:RegisterEvent("CHAT_DISABLED_CHANGED", "Update")
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "Update")
	if C.IS_RETAIL then self:RegisterEvent("SOCIAL_QUEUE_UPDATE", "Update") end
	self:RegisterEvent("TRIAL_STATUS_UPDATE", "Update")
	self:RegisterMessage("SIXUI_CLUTTER_STATE_CHANGED", "Update")
	self:RegisterMessage(GUILD_EVENT, "Update")
	self:ScheduleRepeatingTimer("UpdateGuildRoster", 1)
	self:Update(GUILD_EVENT)
end

function M:IsAvailable()
	return not IsTrialAccount() and C_Club.IsEnabled() and BNConnected() and C_Club.IsRestricted() == Enum.ClubRestrictionReason.None
end

function M:IsReady()
	return #self.clubs > 0
end

function M:Update(event, clubId)
	if event == "SIXSOCIAL_CLUB_UPDATE" then
		if not clubId then
			self.clubs = self:GetClubs()
			self.lastGuildClubId, self.guildClubId = self.guildClubId, Social:GetGuildClubId()

			if self.lastGuildClubId ~= self.guildClubId then
				self.selectedClubId = self.guildClubId
			end

			self:SelectClub(self.selectedClubId)
		elseif clubId ~= self.selectedClubId then
			return
		end
	elseif event == "SIXSOCIAL_GUILD_UPDATE" then
		self.clubs = {Social:GetGuild()}
	elseif event == "SIXUI_CLUTTER_STATE_CHANGED" then
		self.isClutterKeyDown = clubId
	end

	O:Update()
	Tooltip:Refresh()
end

function M:UpdateGuildRoster()
	if Tooltip:IsShown() and self:IsReady() and self.clubs[1].id == self.guildClubId then
		C_GuildInfo.GuildRoster()
	end
end

function M:GetClubs()
	return list(Social:IterateClubs())
end

function M:GetMembers()
	return list(self.clubs[1]:IterateMembers())
end

function M:ScrollDown()
	table.insert(self.clubs, 1, table.remove(self.clubs, #self.clubs))
end

function M:ScrollTo(clubId)
	for index = 1, #self.clubs do
		if self.clubs[1].id ~= clubId then
			self:ScrollDown()
		end
	end
end

function M:ScrollUp()
	table.insert(self.clubs, #self.clubs, table.remove(self.clubs, 1))
end

function M:SelectClub(clubId)
	if (not self:IsReady()) then return end
	if (not clubId) then clubId = self.clubs[1].id end
	self.selectedClubId = clubId
	SetCVar("lastSelectedClubId", clubId)
	C_Club.SetClubPresenceSubscription(clubId)
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnClick(frame, button)
	if button == "RightButton" then
		if not CommunitiesFrame or not CommunitiesFrame.InviteButton or not CommunitiesFrame.InviteButton:IsEnabled() then return end
		CommunitiesFrame.InviteButton:Click()
	elseif button == "MiddleButton" then
		M:SelectClub(M.guildClubId)
		O:Update()
		Tooltip:Refresh()
	else
		ToggleGuildFrame()
	end
end

function O.OnEnter(frame)
	M:UpdateGuildRoster()

	if frame then
		Tooltip:UpdateConfig({anchor = frame})
	end

	Tooltip:Show()
end

function O.OnMouseWheel(frame, value)
	if CommunitiesFrame and CommunitiesFrame:IsShown() then return end
	if not M:IsReady() then return end

	if value > 0 then
		M:ScrollDown()
	else
		M:ScrollUp()
	end

	M:SelectClub()
	O:Update()
	Tooltip:Refresh()
end

function O:Update()
	if not M:IsAvailable() or not M:IsReady() then
		self.text = ""
		self.value = nil
		self.glyph = C.Enum.Glyph.UserSlash
	elseif M.isClutterKeyDown then
		local numOnline  = GetNumOnline(M:GetMembers())

		self.text = numOnline
		self.value = numOnline
		self.glyph = nil
	else
		self.text = ""
		self.value = nil
		self.glyph = C.Enum.Glyph.Users
	end
end

-------------------------------------------------------------------------------
-- Tooltip
-------------------------------------------------------------------------------

local function Header_OnClick(self, key, button)
	A:HandleClick({
		{mouseButton = C.Enum.MouseButton.Left,   callback = function () col, rev = key, nil end},
		{mouseButton = C.Enum.MouseButton.Middle, callback = function () col, rev = nil, nil end},
		{mouseButton = C.Enum.MouseButton.Right,  callback = function () col, rev = key, key end},
	}, button)

	Tooltip:Refresh()
end

function Tooltip:AddMembersHeader()
	if M.isClutterKeyDown then
		local line = self:AddHeader("-", LEVEL, NAME, ZONE, RANK, LABEL_NOTE, DUNGEON_SCORE)

		self:SetCellScript(line, 1, "OnMouseUp", Header_OnClick, "statusOrder")
		self:SetCellScript(line, 2, "OnMouseUp", Header_OnClick, "level")
		self:SetCellScript(line, 3, "OnMouseUp", Header_OnClick, "name")
		self:SetCellScript(line, 4, "OnMouseUp", Header_OnClick, "zone")
		self:SetCellScript(line, 5, "OnMouseUp", Header_OnClick, "rankOrder")
		self:SetCellScript(line, 6, "OnMouseUp", Header_OnClick, "note")
		self:SetCellScript(line, 7, "OnMouseUp", Header_OnClick, "mythicRating")
		self:AddSeparator(C.TOOLTIP_SEPARATOR_HEIGHT)
	else
		self:AddSpannedHeader(M.clubs[1].name or MEMBERS)
	end
end

function Tooltip:AddMembers()
	local members = M:GetMembers()

	if col then
		table.sort(members, SortMembersByColumn)
	else
		table.sort(members)
	end

	Social:UpdateGroups()

	for _, member in next, members do
		if member.status ~= Social.Enum.Status.Offline then
			self:AddMember(member)
		end
	end
end

local function Member_OnClick(self, data, button)
	A:HandleClick({
		{mouseButton = C.Enum.MouseButton.Left, modifierKey = C.Enum.ModifierKey.Alt,     callback = function (m) m:SendInvite() end},
		{mouseButton = C.Enum.MouseButton.Left, modifierKey = C.Enum.ModifierKey.Control, callback = function (m) if m.note then A.SetEditBoxText(m.note) end end},
		{mouseButton = C.Enum.MouseButton.Left, modifierKey = C.Enum.ModifierKey.None,    callback = function (m) m:SendTell() end},
		{mouseButton = C.Enum.MouseButton.Left, modifierKey = C.Enum.ModifierKey.Shift,   callback = function (m) m:SendWho() end},
	}, button, data.member, data.groupGUID)
end

local function Member_OnEnter(self, data)
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	A:ShowGroupTooltip(GameTooltip, data.member.unit, data.groupGUID)
	GameTooltip:Show()
end

local function Member_OnLeave(self)
	if GameTooltip:IsOwned(self) then
		GameTooltip:Hide()
	end
end

function Tooltip:AddMember(member)
	local groupGUID = Social:GetGroupGuid(member.guid, member.unit)
	local levelColor = A.GetDifficultyColor(member.level)
	local mythicRatingColor = C_ChallengeMode.GetDungeonScoreRarityColor(member.mythicRating)
	local nameColor = C.CLASS_TO_COLOR[member.className] or BATTLENET_FONT_COLOR
	local noteColor = NORMAL_FONT_COLOR
	local rankColor = NORMAL_FONT_COLOR
	local statusColor = C.Enum.StatusColor[member.status]
	local statusGlyph = (A.UnitInGroup(member.unit) and C.Enum.Glyph.Check) or (groupGUID and C.Enum.Glyph.Hourglass) or C.Enum.Glyph.Circle
	local zoneColor = NORMAL_FONT_COLOR

	local line = self:AddLine(
		nil,
		M.config.columns.level and member.level ~= 0 and levelColor:WrapTextInColorCode(member.level) or nil,
		M.config.columns.name and member.name and nameColor:WrapTextInColorCode(member.name) or nil,
		M.config.columns.zone and member.zone and zoneColor:WrapTextInColorCode(member.zone) or nil,
		M.config.columns.rank and member.rank and rankColor:WrapTextInColorCode(member.rank) or nil,
		M.config.columns.note and member.note and noteColor:WrapTextInColorCode("[" .. member.note .. "]") or nil,
		M.config.columns.mythicRating and member.mythicRating ~= 0 and mythicRatingColor:WrapTextInColorCode(BreakUpLargeNumbers(member.mythicRating)) or nil
	)

	if M.config.columns.status then
		self:SetCell(line, 1, statusColor:WrapTextInColorCode(statusGlyph), C.GLYPH_SMALL, "CENTER")
		self:SetLineScript(line, "OnMouseUp", Member_OnClick, {member = member, groupGUID = groupGUID})
	end

	if groupGUID then
		self:SetLineScript(line, "OnEnter", Member_OnEnter, {member = member, groupGUID = groupGUID})
		self:SetLineScript(line, "OnLeave", Member_OnLeave)
	end
end

local function Message_OnClick(self, broadcast)
	A.SetEditBoxText(broadcast)
end

function Tooltip:AddMessageOfTheDay()
	local broadcast = M.clubs[1].broadcast

	if broadcast and broadcast ~= "" then
		self:AddSpannedHeader(COMMUNITIES_SETTINGS_MOTD_LABEL)
		self:AddDynamicLine(C.GUILD_UTILITY_COLOR:WrapTextInColorCode(broadcast), Message_OnClick, broadcast, false)
		self:AddSeparator(5, 0, 0, 0, 0)
	end
end

function Tooltip:OnShow()
	if not M:IsAvailable() or not M:IsReady() then
		self:AddSpannedHeader(COMMUNITIES or GUILD)
		self:AddSpannedLine(UNAVAILABLE)
	else
		self:AddMessageOfTheDay()
		self:AddMembersHeader()
		self:AddMembers()
		return true
	end
end

function Tooltip:OnRefresh()
	M:UpdateGuildRoster()
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:SetCVar(name, value)
	if name == "lastSelectedClubId" and self:IsReady() and value ~= self.clubs[1].id then
		self:ScrollTo(value)
	end
end
